const MAPWIDTH = document.getElementById('mapa').offsetWidth;
const MAPHEIGHT = document.getElementById('mapa').offsetHeight;
const GRAFWIDTH = document.getElementById('graficos').offsetWidth;
const GRAFHEIGHT = document.getElementById('graficos').offsetHeight;
const LEGWIDTH = document.getElementById('leyenda').offsetWidth;
const LEGHEIGHT = document.getElementById('leyenda').offsetHeight;

//Archivos de datos
var genderIneq = d3.csv('DesigualdadGenero.csv');
var mapData = d3.json('custom.geo.json');

//SVG principal del mapa
var svg = d3.select("#mapa")
			.append("svg")
    		.attr("width",MAPWIDTH)
    		.attr("height",MAPHEIGHT);

//SVG principal de los graficos secundarios
var svg2 = d3.select("#graficos")
				.append("svg")
				.attr("width", GRAFWIDTH)
				.attr("height", GRAFHEIGHT);

Promise.all([genderIneq, mapData]).then(function(data){

	var genderIneq = data[0];
	var mapData = data[1];

	//Valor inicial de las variables de representacion
	var gen = "M.";
	var varRep = "GII";

	var path = d3.geoPath(d3.geoMercator().fitSize([MAPWIDTH, MAPHEIGHT],mapData));

	//Concatenamos los datos a representar con los datos del mapa y guardamos los colores
	var changeD = changeData(gen, varRep, genderIneq, mapData);
	var color = changeD[0];
	var min = changeD[1];
	var max = changeD[2];

	//Mapa
	var map = svg
    	.append("g")
    	.call(d3.zoom().scaleExtent([0.9,10]).on("zoom", function () {
       map.attr("transform", d3.event.transform)
    }));

    map.attr("width",MAPWIDTH)
    	.attr("height",MAPHEIGHT);

	map.selectAll("path")
		.data(mapData.features)
		.enter()
		.append("path")
		.attr("d", path)
		.style("fill", "white")
		.style("stroke", "black")
		.style("stroke-width", "0.5px")
		.on("mouseover", mouseover)
		.on("mouseout", mouseout)
		.on("click", click)
		.transition()
  		.duration(800)
		.style("fill", changeColor)
		.style("stroke", "transparent")
		.style("stroke-width", "0px");

	//Titulo
	svg.append("text")
		.attr("x", MAPWIDTH/2)
		.attr("y", 20)
		.attr("text-anchor","middle")
		.style("font-weight","bold")
		.style("font-size","25px")
		.text("Desigualdad de Genero por País en 2019");

	//Leyenda
	var legend = d3.select("#leyenda")
						.append("svg")
						.attr("width", LEGWIDTH)
						.attr("height", LEGHEIGHT);

	changeLegend(color, min, max);

	//---Graficos secundarios---
	//Variable de pais pulsado
	var indPais = 24;

	svg2.append("line")
		.attr("x1",0)
		.attr("y1",0)
		.attr("x2",0)
		.attr("y2",GRAFHEIGHT)
		.style("stroke", "black");

	//-Grafico de sectores-
	svg2.append("text")
		.attr("x", GRAFWIDTH/2)
		.attr("y", GRAFHEIGHT/14 + 18)
		.attr("text-anchor","middle")
		.style("font-weight","bold")
		.style("font-size","18px")
		.text("Reparto de Asientos del Parlamento");

	pie = svg2.append("svg")
				.attr("x", (GRAFWIDTH - GRAFWIDTH/2)/2)
				.attr("width", GRAFWIDTH/2)
				.attr("y", GRAFHEIGHT/14 + 18)
				.attr("height", GRAFHEIGHT/4);

	//Creo el radio y la funcion para gestionar los datos para representarlos
	var radius = Math.min(GRAFWIDTH/2-50, GRAFHEIGHT/4-50) / 2;
	var tarta = d3.pie().value(function(d) {return d.value; }).sort(null);

	//Variable de los arcos del grafico
	var arc = d3.arc()
    	.innerRadius(0)
    	.outerRadius(radius - 20);

    //Creo un primer grafico de sectores en blanco para poder hacer la transicion
    var path = pie.append("g")
    				.attr("id", "pie")
			    	.attr("transform", "translate(" + GRAFWIDTH/4 + "," + GRAFHEIGHT/8 + ")")
					.selectAll("path")
					.data(tarta(d3.entries({a: 0, b: 100})))
					.enter().append("path")
					.attr("fill", "purple")
					.attr("d", arc)
					.style("stroke", "black")
					.each(function(d) { this._current = d; })
					.on("mouseover", function(d){
						d3.select(this)
							.style("stroke", "red");
						d3.select("#pie")
							.append("text")
							.attr("id", d.data.key)
							.attr("transform", "translate(" + arc.centroid(d) + ")")
			        		.attr("text-anchor","middle")
							.style("font-size","12px")
							.text(parseFloat(d.value).toFixed(2));
					})
					.on("mouseout", function(d){
						d3.select(this)
			        		.style("stroke", "black");
			        	d3.select("#" + d.data.key).remove();
					});

	//Leyenda del grafico de sectores
	pie.append("rect")
		.attr("x",GRAFWIDTH/2 - "Mujer".length*11)
		.attr("y", 10)
		.attr("width",10)
		.attr("height",10)
		.style("fill","#E64FC3");
	pie.append("text")
		.attr("x",GRAFWIDTH/2 - "Mujer".length*5)
		.attr("y", 20)
		.attr("text-anchor","middle")
		.style("font-size","12px")
		.text("Mujer");

	pie.append("rect")
		.attr("x",GRAFWIDTH/2 - "Hombre".length*9.2)
		.attr("y", 25)
		.attr("width",10)
		.attr("height",10)
		.style("fill","#006FFF");
	pie.append("text")
		.attr("x",GRAFWIDTH/2 - "Hombre".length*4)
		.attr("y", 35)
		.attr("text-anchor","middle")
		.style("font-size","12px")
		.text("Hombre");

	//-De barras 1-
	svg2.append("text")
		.attr("x", GRAFWIDTH/2)
		.attr("y", GRAFHEIGHT/4 + GRAFHEIGHT/14 + 36) 
		.attr("text-anchor","middle")
		.style("font-weight","bold")
		.style("font-size","18px")
		.text("Población con alguna educación secundaria");

	tabular1 = svg2.append("svg")
				.attr("x", (GRAFWIDTH - GRAFWIDTH/2)/2)
				.attr("width", GRAFWIDTH/2)
				.attr("y", GRAFHEIGHT/4 + GRAFHEIGHT/14 + 40)
				.attr("height", GRAFHEIGHT/4);

	//-De barras 2-
	svg2.append("text")
		.attr("x", GRAFWIDTH/2)
		.attr("y", GRAFHEIGHT*2/4 + GRAFHEIGHT/14 + 65)
		.attr("text-anchor","middle")
		.style("font-weight","bold")
		.style("font-size","18px")
		.text("Población Activa");

	tabular2 = svg2.append("svg")
				.attr("x", (GRAFWIDTH - GRAFWIDTH/2)/2)
				.attr("width", GRAFWIDTH/2)
				.attr("y",  GRAFHEIGHT*2/4 + GRAFHEIGHT/14 + 69)
				.attr("height", GRAFHEIGHT/4);

	changeGraphs();

	//Gestion del cambio de variable de representacion
	d3.select("#selecVar")
		.on("change", function(){
			varRep = d3.select("#selecVar").property("value");
			changeD = changeData(gen, varRep, genderIneq, mapData);
			color = changeD[0];
			min = changeD[1];
			max = changeD[2];
			changeLegend(color, min, max);

			//Activo o desactivo el select de genero segun si es un variable con distincion de genero o no
			if(varRep == "AsientosParlamento" | varRep == "Educacion" | varRep == "Pob.Activa"){
				document.getElementById("selecGen").disabled = false;
			}else {
				document.getElementById("selecGen").disabled = true;
			}

			//Vuelvo a pintar el mapa con los nuevos datos
			d3.selectAll("path")
				.data(mapData.features)
				.transition()
		  		.duration(800)
				.style("fill", changeColor)
				.style("stroke", "transparent")
				.style("stroke-width", "0px");
		});

	//Gestion del cambio de genero de representacion
	d3.select("#selecGen")
		.on("change", function(){
			gen = d3.select("#selecGen").property("value");
			changeD = changeData(gen, varRep, genderIneq, mapData);
			color = changeD[0];
			min = changeD[1];
			max = changeD[2];
			changeLegend(color, min, max);

			//Vuelvo a pintar el mapa con los nuevos datos
			d3.selectAll("path")
				.data(mapData.features)
				.transition()
		  		.duration(800)
				.style("fill", changeColor)
				.style("stroke", "transparent")
				.style("stroke-width", "0px");
		});

	//Funcion para elegir el color de cada pais en la representacion
	function changeColor(d){
		var value = d.properties.value;
		if (value >= 0) {
			return color(value);
		} else {
			//Si el valor no esta definido
			return "#bbb";
		}
	}

	//Funcion para gestionar el evento de mouseover de un pais en el mapa
	function mouseover(features){
		var pais = features.properties.name_long;
		//Si el valor no esta definido lo llamamos NaN
		if(features.properties.value == undefined){
			var valor = "NaN";
		}else if(features.properties.value % 1 != 0){ 
			//Si es un decimal recortamos el valor para que sea mas legible
			if(varRep != "GII" ){
				var valor = (features.properties.value).toFixed(2);
			}else{
				var valor = (features.properties.value).toFixed(3);
			}
		}else{
			var valor = features.properties.value;
		}

		//Marco el borde del pais
		d3.select(this).style("stroke", "black")
						.style("stroke-width", "0.5px");

		//Creo el cuadro de info
		var infoBox = d3.select("body")
						.append("div")
						.attr("class", "infoBox")
						.attr("id", "infoBox")
						.style("opacity", 0.8)
		var unit = "";
		if(varRep == "AsientosParlamento" || varRep == "Educacion" || varRep == "Pob.Activa"){
			unit = "%";
		}	
		infoBox.html("<b>Pais</b>: " + pais + "<br><b>Valor de " + varRep + "</b>: " + valor + unit)
			.style("left", d3.event.pageX + "px")
			.style("top", d3.event.pageY + "px");
	}

	//Funcion para gestionar el evento de mouseout de un pais en el mapa
	function mouseout(features){
		d3.select(this).style("stroke", "transparent")
						.style("stroke-width", "0px");
		//Elimino el cuadro de info
		d3.select("#infoBox").remove();
	}

	//Funcion para gestionar el evento de clicado de un pais en el mapa
	function click(position){
		for(var i = 0; i < genderIneq.length; i++){
			if(genderIneq[i].iso_a3 == position.properties.iso_a3){
				indPais = i;
				break;
			}
		}
		changeGraphs();
	}

	/*Funcion para cambiar la leyenda segun la variable de representacion y el genero seleccionados
	pasando como parametro la variable de colores, el valor minimo y el maximo de los datos*/
	function changeLegend(color, min, max){
		let iter = max/100;
		//Elimino la leyenda anterior
		d3.select("#varRep").remove();
		for(var i = 0; i <= 100;i++){
			d3.select("#barrita").remove();
		}
		d3.select("#min").remove();
		d3.select("#max").remove();

		legend.append("text")
				.attr("id","varRep")
				.attr("x",String(varRep).length*4 + 11)
				.attr("y",15)
				.attr("text-anchor","middle")
				.style("font-weight","bold")
				.style("font-size","12px")
				.text(varRep + ":");

		//Creo la leyenda creando 100 rectangulos de distintos colores para simular el degradado
		for (var i = min; i < max; i+=iter){
			legend.append("rect")
					.attr("id","barrita")
					.attr("x", (String(varRep).length*8 + 10) + (LEGWIDTH*0.7-(String(varRep).length*8 + 10))/(max/i))
					.attr("y",4)
					.transition()
					.duration(800)
					.attr("width",(LEGWIDTH*0.7-(String(varRep).length*8 + 10))/(max/iter) + 1)
					.attr("height", "15px")
					.attr("fill", color(i));
		}

		legend.append("text")
				.attr("id","min")
				.attr("x",(String(varRep).length*8 + 10) + (LEGWIDTH*0.7-(String(varRep).length*8 + 10))/(max/iter))
				.attr("y",30)
				.attr("text-anchor","middle")
				.style("font-weight","bold")
				.style("font-size","12px")
				.text(min);

		legend.append("text")
				.attr("id","max")
				.attr("x",(String(varRep).length*8 + 10) + (LEGWIDTH*0.7-(String(varRep).length*8 + 10)))
				.attr("y",30)
				.attr("text-anchor","middle")
				.style("font-weight","bold")
				.style("font-size","12px")
				.text(max);

	}

	/*Funcion para cambiar los graficos secundarios segun el pais clicado guardado por su 
	su indice en la variable indPais*/
	function changeGraphs(){
		d3.select("#Pais").remove();
		
		svg2.append("text")
			.attr("id","Pais")
			.attr("x", String(genderIneq[indPais].Pais).length*5 + 10)
			.attr("y", 20)
			.attr("text-anchor","middle")
			.style("font-weight","bold")
			.style("font-size","20px")
			.style("text-decoration", "underline")
			.text(genderIneq[indPais].Pais);

		d3.select("#Ranking").remove();
		
		svg2.append("text")
			.attr("id","Ranking")
			.attr("x", GRAFWIDTH-90)
			.attr("y", 20)
			.attr("text-anchor","middle")
			.style("font-weight","bold")
			.style("font-size","20px")
			.text("Ranking GII: " + ((genderIneq[indPais].Ranking!="") ? genderIneq[indPais].Ranking : NaN));

		//Grafico de sectores
		var dataPie = {Mujer: genderIneq[indPais]["AsientosParlamento"],Hombre: 100 - genderIneq[indPais]["AsientosParlamento"]};
		var colorPie = d3.scaleOrdinal()
						 .domain(dataPie)
						 .range(["#E64FC3", "#006FFF"]);

		//Gestiono los datos para poder usarlos para el grafico
		var data_ready = tarta(d3.entries(dataPie));

		//A partir de los datos sobrescribo el grafico de sectores haciendo la transicion
		path = path.data(data_ready).attr("fill", function(d, i) { return colorPie(i); });
		path.transition()
			.duration(750)
			.attrTween("d", function(a){
				var i = d3.interpolate(this._current, a);
				this._current = i(0);
				return function(t){
					return arc(i(t));
				}
			});

		//Barras 1
		var dataTab1 = [{Genero: "Mujer", Valor: genderIneq[indPais]["M.Educacion"]}, {Genero: "Hombre", Valor: genderIneq[indPais]["H.Educacion"]}];
		
        d3.select("#tab1").remove();

  		//Si no hay datos no creo el grafico
        if(dataTab1[0].Valor != "" & dataTab1[1].Valor != ""){
			var g1 = tabular1.append("g")
							.attr("id", "tab1")
	        				.attr("transform", "translate(" + 27 + "," + 5 + ")");
	        graficoBarras(g1, dataTab1, "#tab1");
	    }else {
	    	var g1 = tabular1.append("text")
	    					.attr("id", "tab1")
	    					.attr("x", GRAFWIDTH/4)
	    					.attr("y", GRAFHEIGHT/8)
	    					.attr("text-anchor","middle")
							.style("font-weight","bold")
							.style("font-size","20px")
							.text("Sin datos");
	    }

        //Barras 2
		var dataTab2 = [{Genero: "Mujer", Valor: genderIneq[indPais]["M.Pob.Activa"]}, {Genero: "Hombre", Valor: genderIneq[indPais]["H.Pob.Activa"]}];
		
        d3.select("#tab2").remove();

        if(dataTab2[0].Valor != "" || dataTab2[1].Valor != ""){
			var g2 = tabular2.append("g")
							.attr("id", "tab2")
	        				.attr("transform", "translate(" + 27 + "," + 5 + ")");

	        graficoBarras(g2, dataTab2, "#tab2")
	    }else {
	    	var g2 = tabular2.append("text")
	    					.attr("id", "tab2")
	    					.attr("x", GRAFWIDTH/4)
	    					.attr("y", GRAFHEIGHT/8)
	    					.attr("text-anchor","middle")
							.style("font-weight","bold")
							.style("font-size","20px")
							.text("Sin datos");
	    }
	}

	/*Funcion general para crear un diagrama de barras a partir del objeto svg, los datos y el id*/
	function graficoBarras(g, dataTab, tabId){
		//Creo los datos para hacer los ejes
		var x = d3.scaleBand().range([0, GRAFWIDTH/2 - 27]).padding(0.5);
		var y = d3.scaleLinear().range([GRAFHEIGHT/4 - 22, 0]);
		x.domain(dataTab.map(function(d) { return d.Genero; }));
        y.domain([0, 100]);

        //Eje x
		g.append("g")
	        .attr("transform", "translate(0," + (GRAFHEIGHT/4 - 22) + ")")
	        .call(d3.axisBottom(x));

	    //Eje y
	    g.append("g")
	        .call(d3.axisLeft(y).ticks(10))
	    	.append("text")
	        .attr("transform", "rotate(-90)")
	        .attr("y", 6)
	        .attr("text-anchor", "end");

	    //Creo el grafico de barras
	    g.selectAll(".bar")
	        .data(dataTab)
	        .enter().append("rect")
	        .attr("x", function(d) { return x(d.Genero); })
	        .attr("width", x.bandwidth())
	        .style("fill", function(d){return (d.Genero == "Mujer") ? "#E64FC3" : "#006FFF"})
	        .attr("height", function(d) { return 0; })
	        .attr("y", function(d) { return GRAFHEIGHT/4 -22; });

	    g.selectAll("rect")
	      	.on("mouseover", function(d){
	        	d3.select(this)
	        		.style("stroke", "red");
	        	d3.select(tabId)
	        		.append("text")
	        		.attr("id", d.Genero + "Val")
	        		.attr("x", parseFloat(d3.select(this).attr("x")) + x.bandwidth()/2 )
	        		.attr("y", parseFloat(d3.select(this).attr("y")) + parseFloat(d3.select(this).attr("height"))/2)
	        		.attr("text-anchor","middle")
					.style("font-size","12px")
	        		.text(parseFloat(d.Valor).toFixed(2));
	        })
	        .on("mouseout", function(d){
	        	d3.select(this)
	        		.style("stroke", "transparent");
	        	d3.select("#" + d.Genero + "Val").remove();
	        })
	        .transition()
	  		.duration(800)
	  		.attr("height", function(d) { return GRAFHEIGHT/4 - 22 - y(d.Valor); })
	        .attr("y", function(d) { return y(d.Valor); });
	}

});

/*Funcion que a partir del genero y la variable de representacion elegidas concatena los datos 
con los datos del mapa segun el iso_a3, elige los colores y genera una variable de colores para
representar el mapa*/
function changeData(gen, varRep, genderIneq, mapData){
	var low = "purple"; var high = "#E5CCFF";
	var min = 999; var max = 0;
	var exp = 1;
	//Elijo los colores segun la variable y el genero elegidos 
	switch(varRep){
		case "GII": max = 1; min = 0; break;
		case "Mort.Maternal": low = "#FFC0C0"; high = "red"; exp = 0.3; break;
		case "Nac.Adolescentes": low = "#FFC0C0"; high = "red"; exp = 0.7; break;
		case "AsientosParlamento":
			max = 100; min = 0; exp = 0.8;
			if(gen=="M."){
				low = "#FFE5FA"; high = "#E64FC3";
			}else{
				low = "#E4F0FF"; high = "#006FFF";
			}
			break;
		case "Educacion": varRep = String(gen).concat(String(varRep));
			max = 100; min = 0; exp = 1.5;
			if(gen=="M."){
				low = "#FFE5FA"; high = "#E64FC3";
			}else{
				low = "#E4F0FF"; high = "#006FFF";
			}
			break;
		case "Pob.Activa": varRep = String(gen).concat(String(varRep));
			max = 100; min = 0; exp = 1.5;
			if(gen=="M."){
				low = "#FFE5FA"; high = "#E64FC3";
			}else{
				low = "#E4F0FF"; high = "#006FFF";
			}
			break;
	}
	//Eliminamos los datos anteriores que pudiese haber
	for (var i = 0; i < mapData.features.length; i++){
		mapData.features[i].properties.value = undefined;
	}
	//Concatenamos los datos al mapData
	for (var i = 0; i < genderIneq.length; i++) {
		dataISO = genderIneq[i].iso_a3;
		//Cogemos el valor varRep y lo convertimos a float
		if (varRep == "AsientosParlamento" & gen == "H."){
			var dataValue = 100 - parseFloat(genderIneq[i][varRep]);
		}else{
			var dataValue = parseFloat(genderIneq[i][varRep]);
		}
		//Buscamos valores maximos y minimos en los datos
		if(min > dataValue){
			min = dataValue;
		}
		if(max < dataValue){
			max = dataValue;
		}
		//Buscamos el iso_a3 correspondiente en el json mapData
		for (var j = 0; j < mapData.features.length; j++) {
			var mapISO = mapData.features[j].properties.iso_a3;
			if (dataISO==mapISO) {
				if(mapData.features[j].properties.value == undefined){
					mapData.features[j].properties.value = dataValue;
					break;
				}
			}
		}
	}
	//generamos la variable de colores
	var color = d3.scalePow()
                .exponent(exp)
                .domain([min,max])
                .range([low,high]);
    return [color, min, max];
}